@@ -74,7 +74,8 @@ impl TestBounds {
             Self::From(start) if start > &len || !control.is_char_boundary(*start) => true,
             Self::To(end) if end > &len || !control.is_char_boundary(*end) => true,
             Self::Inclusive(start, end)
-                if start > end
+                if *end == usize::max_value()
+                    || *start > (end + 1)
                     || start > &len
                     || end > &len
                     || !control.is_char_boundary(*start)
@@ -145,8 +146,8 @@ pub enum Action {
     Clear,
     IntoString,
     Retain(String),
-    // Drain(TestBounds),
-    // ReplaceRange(TestBounds, String),
+    Drain(TestBounds),
+    ReplaceRange(TestBounds, String),
 }
 
 impl Action {
@@ -228,114 +229,128 @@ impl Action {
                 let f = |ch| filter.contains(ch);
                 control.retain(f);
                 subject.retain(f);
-            } // FIXME: skipping `drain` and `replace_range` tests, pending https://github.com/rust-lang/rust/issues/72237
-              // Self::Drain(range) => {
-              //     if range.should_panic(&control) {
-              //         assert_panic(|| match range {
-              //             TestBounds::Range(start, end) => {
-              //                 (control.drain(start..end), subject.drain(start..end))
-              //             }
-              //             TestBounds::From(start) => (control.drain(start..), subject.drain(start..)),
-              //             TestBounds::To(end) => (control.drain(..end), subject.drain(..end)),
-              //             TestBounds::Full => (control.drain(..), subject.drain(..)),
-              //             TestBounds::Inclusive(start, end) => {
-              //                 (control.drain(start..=end), subject.drain(start..=end))
-              //             }
-              //             TestBounds::ToInclusive(end) => {
-              //                 (control.drain(..=end), subject.drain(..=end))
-              //             }
-              //         })
-              //     } else {
-              //         let (control_iter, subject_iter) = match range {
-              //             TestBounds::Range(start, end) => {
-              //                 (control.drain(start..end), subject.drain(start..end))
-              //             }
-              //             TestBounds::From(start) => (control.drain(start..), subject.drain(start..)),
-              //             TestBounds::To(end) => (control.drain(..end), subject.drain(..end)),
-              //             TestBounds::Full => (control.drain(..), subject.drain(..)),
-              //             TestBounds::Inclusive(start, end) => {
-              //                 (control.drain(start..=end), subject.drain(start..=end))
-              //             }
-              //             TestBounds::ToInclusive(end) => {
-              //                 (control.drain(..=end), subject.drain(..=end))
-              //             }
-              //         };
-              //         let control_result: String = control_iter.collect();
-              //         let subject_result: String = subject_iter.collect();
-              //         assert_eq!(control_result, subject_result);
-              //     }
-              // }
-              // Self::ReplaceRange(range, string) => {
-              //     if range.should_panic(&control) {
-              //         assert_panic(|| match range {
-              //             TestBounds::Range(start, end) => {
-              //                 control.replace_range(start..end, &string);
-              //                 subject.replace_range(start..end, &string);
-              //             }
-              //             TestBounds::From(start) => {
-              //                 control.replace_range(start.., &string);
-              //                 subject.replace_range(start.., &string);
-              //             }
-              //             TestBounds::To(end) => {
-              //                 control.replace_range(..end, &string);
-              //                 subject.replace_range(..end, &string);
-              //             }
-              //             TestBounds::Full => {
-              //                 control.replace_range(.., &string);
-              //                 subject.replace_range(.., &string);
-              //             }
-              //             TestBounds::Inclusive(start, end) => {
-              //                 control.replace_range(start..=end, &string);
-              //                 subject.replace_range(start..=end, &string);
-              //             }
-              //             TestBounds::ToInclusive(end) => {
-              //                 control.replace_range(..=end, &string);
-              //                 subject.replace_range(..=end, &string);
-              //             }
-              //         })
-              //     } else {
-              //         match range {
-              //             TestBounds::Range(start, end) => {
-              //                 control.replace_range(start..end, &string);
-              //                 subject.replace_range(start..end, &string);
-              //             }
-              //             TestBounds::From(start) => {
-              //                 control.replace_range(start.., &string);
-              //                 subject.replace_range(start.., &string);
-              //             }
-              //             TestBounds::To(end) => {
-              //                 control.replace_range(..end, &string);
-              //                 subject.replace_range(..end, &string);
-              //             }
-              //             TestBounds::Full => {
-              //                 control.replace_range(.., &string);
-              //                 subject.replace_range(.., &string);
-              //             }
-              //             TestBounds::Inclusive(start, end) => {
-              //                 control.replace_range(start..=end, &string);
-              //                 subject.replace_range(start..=end, &string);
-              //             }
-              //             TestBounds::ToInclusive(end) => {
-              //                 control.replace_range(..=end, &string);
-              //                 subject.replace_range(..=end, &string);
-              //             }
-              //         }
-              //     }
-              // }
+            }
+            Self::Drain(range) => {
+                // FIXME: ignoring inclusive bounds at usize::max_value(), pending https://github.com/rust-lang/rust/issues/72237
+                match range {
+                    TestBounds::Inclusive(_, end) if end == usize::max_value() => return,
+                    TestBounds::ToInclusive(end) if end == usize::max_value() => return,
+                    _ => {}
+                }
+                if range.should_panic(&control) {
+                    assert_panic(|| match range {
+                        TestBounds::Range(start, end) => {
+                            (control.drain(start..end), subject.drain(start..end))
+                        }
+                        TestBounds::From(start) => (control.drain(start..), subject.drain(start..)),
+                        TestBounds::To(end) => (control.drain(..end), subject.drain(..end)),
+                        TestBounds::Full => (control.drain(..), subject.drain(..)),
+                        TestBounds::Inclusive(start, end) => {
+                            (control.drain(start..=end), subject.drain(start..=end))
+                        }
+                        TestBounds::ToInclusive(end) => {
+                            (control.drain(..=end), subject.drain(..=end))
+                        }
+                    })
+                } else {
+                    let (control_iter, subject_iter) = match range {
+                        TestBounds::Range(start, end) => {
+                            (control.drain(start..end), subject.drain(start..end))
+                        }
+                        TestBounds::From(start) => (control.drain(start..), subject.drain(start..)),
+                        TestBounds::To(end) => (control.drain(..end), subject.drain(..end)),
+                        TestBounds::Full => (control.drain(..), subject.drain(..)),
+                        TestBounds::Inclusive(start, end) => {
+                            (control.drain(start..=end), subject.drain(start..=end))
+                        }
+                        TestBounds::ToInclusive(end) => {
+                            (control.drain(..=end), subject.drain(..=end))
+                        }
+                    };
+                    let control_result: String = control_iter.collect();
+                    let subject_result: String = subject_iter.collect();
+                    assert_eq!(control_result, subject_result);
+                }
+            }
+            Self::ReplaceRange(range, string) => {
+                // FIXME: ignoring inclusive bounds at usize::max_value(), pending https://github.com/rust-lang/rust/issues/72237
+                match range {
+                    TestBounds::Inclusive(_, end) if end == usize::max_value() => return,
+                    TestBounds::ToInclusive(end) if end == usize::max_value() => return,
+                    _ => {}
+                }
+                if range.should_panic(&control) {
+                    assert_panic(|| match range {
+                        TestBounds::Range(start, end) => {
+                            control.replace_range(start..end, &string);
+                            subject.replace_range(start..end, &string);
+                        }
+                        TestBounds::From(start) => {
+                            control.replace_range(start.., &string);
+                            subject.replace_range(start.., &string);
+                        }
+                        TestBounds::To(end) => {
+                            control.replace_range(..end, &string);
+                            subject.replace_range(..end, &string);
+                        }
+                        TestBounds::Full => {
+                            control.replace_range(.., &string);
+                            subject.replace_range(.., &string);
+                        }
+                        TestBounds::Inclusive(start, end) => {
+                            control.replace_range(start..=end, &string);
+                            subject.replace_range(start..=end, &string);
+                        }
+                        TestBounds::ToInclusive(end) => {
+                            control.replace_range(..=end, &string);
+                            subject.replace_range(..=end, &string);
+                        }
+                    })
+                } else {
+                    match range {
+                        TestBounds::Range(start, end) => {
+                            control.replace_range(start..end, &string);
+                            subject.replace_range(start..end, &string);
+                        }
+                        TestBounds::From(start) => {
+                            control.replace_range(start.., &string);
+                            subject.replace_range(start.., &string);
+                        }
+                        TestBounds::To(end) => {
+                            control.replace_range(..end, &string);
+                            subject.replace_range(..end, &string);
+                        }
+                        TestBounds::Full => {
+                            control.replace_range(.., &string);
+                            subject.replace_range(.., &string);
+                        }
+                        TestBounds::Inclusive(start, end) => {
+                            control.replace_range(start..=end, &string);
+                            subject.replace_range(start..=end, &string);
+                        }
+                        TestBounds::ToInclusive(end) => {
+                            control.replace_range(..=end, &string);
+                            subject.replace_range(..=end, &string);
+                        }
+                    }
+                }
+            }
         }
     }
 }
 
 fn assert_invariants<Mode: SmartStringMode>(control: &str, subject: &SmartString<Mode>) {
-    assert_eq!(control, subject.as_str());
     assert_eq!(control.len(), subject.len());
-    assert_eq!(
-        subject.is_inline(),
-        subject.len() <= Mode::MAX_INLINE,
-        "len {} should be inline (MAX_INLINE = {}) but was boxed",
-        subject.len(),
-        Mode::MAX_INLINE
-    );
+    assert_eq!(control, subject.as_str());
+    if Mode::DEALLOC {
+        assert_eq!(
+            subject.is_inline(),
+            subject.len() <= Mode::MAX_INLINE,
+            "len {} should be inline (MAX_INLINE = {}) but was boxed",
+            subject.len(),
+            Mode::MAX_INLINE
+        );
+    }
     assert_eq!(
         control.partial_cmp(&"ordering test".to_string()),
         subject.partial_cmp("ordering test")
@@ -361,8 +376,9 @@ pub fn test_ordering<Mode: SmartStringMode>(left: String, right: String) {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use crate::{Compact, Prefixed};
+    use super::{Action::*, Constructor::*, TestBounds::*, *};
+
+    use crate::{Compact, LazyCompact};
 
     proptest! {
         #[test]
@@ -371,8 +387,8 @@ mod tests {
         }
 
         #[test]
-        fn proptest_everything_prefixed(constructor: Constructor, actions: Vec<Action>) {
-            test_everything::<Prefixed>(constructor, actions);
+        fn proptest_everything_lazycompact(constructor: Constructor, actions: Vec<Action>) {
+            test_everything::<LazyCompact>(constructor, actions);
         }
 
         #[test]
@@ -381,14 +397,14 @@ mod tests {
         }
 
         #[test]
-        fn proptest_ordering_prefixed(left: String, right: String) {
-            test_ordering::<Prefixed>(left,right)
+        fn proptest_ordering_lazycompact(left: String, right: String) {
+            test_ordering::<LazyCompact>(left,right)
         }
     }
 
     #[test]
     fn must_panic_on_insert_outside_char_boundary() {
-        test_everything::<Prefixed>(
+        test_everything::<Compact>(
             Constructor::FromString("a0 A୦a\u{2de0}0 🌀Aa".to_string()),
             vec![
                 Action::Push(' '),
@@ -407,7 +423,7 @@ mod tests {
 
     #[test]
     fn must_panic_on_out_of_bounds_range() {
-        test_everything::<Prefixed>(
+        test_everything::<Compact>(
             Constructor::New,
             vec![Action::Slice(TestBounds::Range(0, 13764126361151078400))],
         );
@@ -415,7 +431,7 @@ mod tests {
 
     #[test]
     fn must_not_promote_before_insert_succeeds() {
-        test_everything::<Prefixed>(
+        test_everything::<Compact>(
             Constructor::FromString("ኲΣ A𑒀a ®Σ a0🠀  aA®A".to_string()),
             vec![Action::Insert(21, ' ')],
         );
@@ -423,46 +439,58 @@ mod tests {
 
     #[test]
     fn must_panic_on_slice_outside_char_boundary() {
-        test_everything::<Prefixed>(
+        test_everything::<Compact>(
             Constructor::New,
             vec![Action::Push('Ь'), Action::Slice(TestBounds::ToInclusive(0))],
         )
     }
 
-    #[test]
-    fn must_compare_correctly_with_different_fragment_char_counts() {
-        test_ordering::<Prefixed>("\u{1b}\u{7be}\nJ\\#\u{7be}J\\\no\u{7be}\n\n[\n\u{2}\n\n\u{11}C\u{0}\u{0}\u{0}A\n\n[\n\u{2}\n\n\u{11}C\u{0}A\n\u{1a}\n\u{7be}JC\u{11}\u{10}C\u{0}[\u{2}\u{1b}\u{7be}\nJ\\XX".to_string(),
-            "\u{1b}\u{7be}\nJ\\\u{7be}\n\n[\n\u{2}\n\n\u{11}C\u{0}\u{0}\u{0}A\n\n[\n\u{2}\n\n\u{11}C\u{0}A\n\u{1a}\n\u{7be}JC\u{11}\u{10}C\u{0}[\u{2}\u{1b}\u{7be}\nJ\\XX\u{1b}\u{7be}\nJ\\#\u{7be}J\\\no\u{7be}\n\n[\n\u{2}\n\n\u{11}C\u{0}\u{0}\u{0}A\n\n[\n\u{2}\n\n\u{11}C\u{0}A\n\u{1a}\n\u{7be}JC\u{11}\u{10}C\u{0}[\u{2}\u{1b}\u{7be}\nJ\\XXXXXXXXX\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}\u{1f}XXXXXXXXXXXXXXXXXXXXXXXXX".to_string());
-    }
-
     #[test]
     fn dont_panic_when_inserting_a_string_at_exactly_inline_capacity() {
         let string: String = (0..Compact::MAX_INLINE).map(|_| '\u{0}').collect();
         test_everything::<Compact>(Constructor::New, vec![Action::InsertStr(0, string)])
     }
 
-    // #[test]
-    // fn drain_a_string_properly() {
-    //     test_everything::<Compact>(
-    //         Constructor::New,
-    //         vec![Action::Push('¡'), Action::Drain(TestBounds::Full)],
-    //     )
-    // }
-
     #[test]
     #[should_panic]
     fn drain_bounds_integer_overflow_must_panic() {
-        // test_everything::<Compact>(
-        //     Constructor::FromString("מ∢∢∢∢∢∢∢∢".to_string()),
-        //     vec![Action::Drain(TestBounds::ToInclusive(usize::max_value()))],
-        // )
         let mut string = SmartString::<Compact>::from("מ");
         string.drain(..=usize::max_value());
     }
 
     #[test]
-    fn string_drain_overflow() {
-        let mut string = String::new();
-        string.drain(..=usize::max_value());
+    fn shouldnt_panic_on_inclusive_range_end_one_less_than_start() {
+        test_everything::<Compact>(
+            Constructor::FromString("\'\'\'\'\'[[[[[[[[[[[-[[[[[[[[[[[[[[[[[[[[[[".to_string()),
+            vec![Action::Slice(TestBounds::Inclusive(1, 0))],
+        )
+    }
+
+    #[test]
+    fn drain_over_inline_boundary() {
+        test_everything::<Compact>(
+            FromString((0..24).map(|_| 'x').collect()),
+            vec![Drain(Range(0, 1))],
+        )
+    }
+
+    #[test]
+    fn drain_wrapped_shouldnt_drop_twice() {
+        test_everything::<Compact>(
+            FromString((0..25).map(|_| 'x').collect()),
+            vec![Drain(Range(0, 1))],
+        )
+    }
+
+    #[test]
+    fn fail() {
+        let value = "fo\u{0}\u{0}\u{0}\u{8}\u{0}\u{0}\u{0}\u{0}____bbbbb_____bbbbbbbbb";
+        let mut control = String::from(value);
+        let mut string = SmartString::<Compact>::from(value);
+        control.drain(..=0);
+        string.drain(..=0);
+        let control_smart: SmartString<Compact> = control.into();
+        assert_eq!(control_smart, string);
+        assert_eq!(Ordering::Equal, string.cmp(&control_smart));
     }
 }
